/*
 *  shouldrun.m
 *  launcharbiter
 *
 *  Created by Graham Lee on 30/09/2005.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */

#import "shouldrun.h"
#import <Foundation/Foundation.h>
#import <unistd.h>
#import <mach/mach_init.h>
#import <IOKit/IOKitLib.h>
#import <IOKit/pwr_mgt/IOPMLib.h>
#import <IOKit/pwr_mgt/IOPM.h>
#import <IOKit/IOReturn.h>

bool arbiterisonbattery(void);
bool arbiteruserisdenied(NSDictionary *gd, NSDictionary *ld);

arbiter_sr shouldrun(const char *bn, NSDictionary *dict)
{
	arbiter_flags flags;
	NSDictionary *globalDict,*localDict;
	
	memset(&flags,0,sizeof(flags));
	
	if(nil!=dict)
	{
		//parse options dictionary
		if(nil!=(globalDict=[dict objectForKey:@"GlobalLaunchArbiterPolicy"]))
		{
			//global policy settings
			flags.battery=(1 == [[globalDict objectForKey:@"NotOnBattery"] intValue])?1:0;
			if(nil!=[globalDict objectForKey:@"AllowUsers"] || nil!=[globalDict objectForKey:@"DenyUsers"]) flags.allowdenyuser=1;
		}
		if(nil!=(localDict=[dict objectForKey:[NSString stringWithCString:bn]]))
		{
			//policy for this program
			if(0!=[[localDict objectForKey:@"NotOnBattery"] intValue]) flags.battery=1;
			if(nil!=[localDict objectForKey:@"AllowUsers"] || nil!=[localDict objectForKey:@"DenyUsers"]) flags.allowdenyuser=1;
		}
	}
	
	// here's where the policy gets decided
	if(flags.battery)
		if(arbiterisonbattery())
			return ARBITER_VETO_BATTERY;
	if(flags.allowdenyuser)
		if(arbiteruserisdenied(globalDict,localDict))
			return ARBITER_VETO_USERDENIED;
	
	return ARBITER_RUN;
}

bool arbiterisonbattery(void)
{
	NSArray *ary;
	static mach_port_t master=0;
	static io_connect_t pmcon=0;
	
	IOMasterPort(bootstrap_port, &master);
	pmcon=IOPMFindPowerManagement(master);
	
	if(IOPMCopyBatteryInfo(master,&ary)==kIOReturnSuccess)
	{
		NSDictionary *dict;
		NSNumber *num;
		int flags;
		
		dict=[ary objectAtIndex:0];
		num=[dict valueForKey:[NSString stringWithCString:kIOBatteryFlagsKey]];
		flags=[num longValue];
		
		[ary release];
		if(!(flags & kIOBatteryChargerConnect))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	return false;
}

bool arbiteruserisdenied(NSDictionary *gd, NSDictionary *ld)
{
	/* the policy here is...
	 * 1 all users are allowed
	 * 2 if there exists a local AllowUsers, everyone except them is denied
	 * 3 if there exists a local DenyUsers, they are denied
	 * 4 if a match has explicitly been made, stop here
	 * 5 do 2-3 for the global AllowUsers and DenyUsers
	 *
	 * ergo, if you're in both AllowUsers *and* DenyUsers in the same scope, you're denied
	 */
	bool amallowed=true/*1*/, didmatch=false;
	int i;
	NSString *logName;
	char *Logname;
	NSArray *currentArray;
	
	Logname=getlogin();
	logName=[NSString stringWithCString:Logname];
	if(nil!=(currentArray=[ld objectForKey:@"AllowUsers"]))
	{
		/*2*/
		amallowed=false;
		for(i=0;i<[currentArray count];i++)
		{
			if([logName isEqualToString:[currentArray objectAtIndex:i]])
			{
				amallowed=true;
				didmatch=true;
			}
		}
	}
	if(nil!=(currentArray=[ld objectForKey:@"DenyUsers"]))
	{
		/*3*/
		for(i=0;i<[currentArray count];i++)
		{
			if([logName isEqualToString:[currentArray objectAtIndex:i]])
			{
				amallowed=false;
				didmatch=true;
			}
		}
	}
	if(true==didmatch)/*4*/
	{
		return !amallowed;
	}
	
	/*5*/
	if(nil!=(currentArray=[gd objectForKey:@"AllowUsers"]))
	{
		amallowed=false;
		for(i=0;i<[currentArray count];i++)
		{
			if([logName isEqualToString:[currentArray objectAtIndex:i]])
				amallowed=true;
		}
	}
	if(nil!=(currentArray=[gd objectForKey:@"DenyUsers"]))
	{
		for(i=0;i<[currentArray count];i++)
		{
			if([logName isEqualToString:[currentArray objectAtIndex:i]])
				amallowed=false;
		}
	}
	return !amallowed;
}